/***************************************************************************
 *   Copyright (C) 2005 by Sebastian Beck                                  *
 *   sebastian.bw@freenet.de                                                    
  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "historyintinput.h"

#include <qpushbutton.h>
#include <qlineedit.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qwidget.h>
#include <qstring.h> 
#include <qframe.h>
#include <qvalidator.h>


unsigned char img0_intinputwidget[] = 
	{ 
	0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a, 0x00, 0x00, 0x00, 0x0d,
	0x49, 0x48, 0x44, 0x52, 0x00, 0x00, 0x00, 0x09, 0x00, 0x00, 0x00, 0x0b,
	0x08, 0x06, 0x00, 0x00, 0x00, 0xad, 0x59, 0xa7, 0x1b, 0x00, 0x00, 0x00,
	0x36, 0x49, 0x44, 0x41, 0x54, 0x18, 0x95, 0x63, 0x64, 0x60, 0x60, 0xf8,
	0xcf, 0x40, 0x37, 0xc0, 0x88, 0xc4, 0xc6, 0x66, 0x2d, 0x23, 0x03, 0x03,
	0x03, 0x03, 0x13, 0x31, 0x26, 0x11, 0xa5, 0x88, 0x11, 0x87, 0x35, 0x58,
	0xdd, 0x84, 0x4f, 0x21, 0x23, 0x51, 0x0e, 0x67, 0x44, 0x13, 0xfc, 0x8f,
	0xae, 0x00, 0x17, 0xc0, 0x30, 0x11, 0x00, 0xa5, 0xb1, 0x07, 0x08, 0xa4,
	0x16, 0xd6, 0x63, 0x00, 0x00, 0x00, 0x00, 0x49, 0x45, 0x4e, 0x44, 0xae,
	0x42, 0x60, 0x82
	};
	
/**
 *Creates a HistoryIntInput object with inputs in the given range and including a popup button right hand.
 * @param parent 
 * @param v 
 * @param prec 
 * @param min 
 * @param max 
 * @param ifa 
 * @param name 
 */
HistoryIntInput::HistoryIntInput( QWidget *parent, int v, uint prec, int min, int max, const char *name ): QFrame(parent, name )
	{
	Value = v;
	MinValue = min;
	MaxValue = max;
	setwidgets( name );
	}

/**
 * Creates also a HistoryIntInput object with this default parameters: Value = 0.00, precision = 2, MinValue = 0, MaxValue = 9999 and including a popup button right hand.
 * @param parent 
 * @param name 
 */
HistoryIntInput::HistoryIntInput( QWidget *parent, const char *name ): QFrame( parent, name )
	{
	Value = 0;
	MinValue = 0;
	MaxValue = 9999;
	setwidgets( name );
	}

/**
 *Sets up all objects and the layout of HistoryIntInput.
 * @param name 
 */
void HistoryIntInput::setwidgets( const char *name )
	{
	QImage img;
	
	cb = new KCompletionBox( this );
	cb->hide();
	cb->setMinimumWidth( 20 );
	img.loadFromData( img0_intinputwidget, sizeof( img0_intinputwidget ), "PNG" );
	image0 = img;
	
	if ( !name )
		setName( "DoubleInputWidget" );
	IntInputWidgetLayout = new QGridLayout( this, 1, 1, 1, 0, "DoubleInputWidgetLayout");
	
	
	pushButton = new QPushButton( this, "pushButton" );
	pushButton->setSizePolicy( QSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred, 0, 0, pushButton->sizePolicy().hasHeightForWidth() ) );
	pushButton->setMinimumSize( QSize( 11, 0 ) );
	pushButton->setMaximumSize( QSize( 11, 32767 ) );
	pushButton->setFocusPolicy( QPushButton::NoFocus );
	pushButton->setAutoMask( TRUE );
	pushButton->setPixmap( image0 );
	pushButton->setFlat( TRUE );
	IntInputWidgetLayout->addWidget( pushButton, 0, 1 );
	
	lineEdit = new QLineEdit( this, "lineEdit" );
	lineEdit->setMinimumSize( QSize( 0, 0 ) );
	lineEdit->setMaximumSize( QSize( 32767, 32767 ) );
	lineEdit->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding, 0, 0, lineEdit->sizePolicy().hasHeightForWidth() ) );
	lineEdit->setFrame( FALSE );
	lineEdit->setAlignment( int( QLineEdit::AlignRight ) );
	IntInputWidgetLayout->addWidget( lineEdit, 0, 0 );
	
	setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );
	languageChange();
	resize( QSize( 20, 11 ).expandedTo(minimumSizeHint()) );
	clearWState( WState_Polished );

	connect( pushButton, SIGNAL( clicked() ), this, SLOT( popup() ) );
	connect( lineEdit, SIGNAL( returnPressed() ), this, SLOT( registerValue() ) );
	connect( lineEdit, SIGNAL( lostFocus() ), this, SLOT( registerValue() ) );
	connect( cb, SIGNAL( activated( const QString& ) ), this, SLOT( insert( const QString& ) ) ) ;

	setTabOrder( lineEdit, pushButton );
	
	iv = new QIntValidator( MinValue, MaxValue, lineEdit );
	lineEdit->setValidator( iv );
	}

HistoryIntInput::~HistoryIntInput()
	{
	}

/**
 * Registers a Value of lineEdit to the popup widget and resets that value in lineEdit.
 * Before you read out a value you have to call this function, to make sure the value displayed is used. This is because of signal handle mistakes of QLineEdit. To do this can be obsoleted with newer versions of QT.
 * This is called from SIGNAL( returnPressed() ) and SIGNAL( lostFocus() ) of lineEdit.
 */
void HistoryIntInput::registerValue()
	{
	Value = lineEdit->displayText().toInt();
	
	if(  ! cb->items().contains( QString::number( Value ) ) )
		cb->insertItem( QString::number( Value ) );
	
	lineEdit->setText( QString::number( Value ) );
	}

/**
 * This is not really needed. You can change "." to "," validation here.
 */
void HistoryIntInput::languageChange()
	{
	pushButton->setText( QString::null );
	lineEdit->setText( tr2i18n( QString::number( Value ) ) );
	}

void HistoryIntInput::popup()
	{
	cb->popup();
	}

/**
 * Inserts the value selected in the popup widget.
 * Called from SIGNAL( activated( const QString& ) ) of popup widget.
 * @param st 
 */
void HistoryIntInput::insert( const QString& st )
	{
	lineEdit->setText( st );
	}

/**
 * Sets HistoryIntInput's Value.
 * @param d 
 */
void HistoryIntInput::setValue( int d )
	{
	Value = d;
	lineEdit->setText( QString::number( Value ) );

	if(  ! cb->items().contains( QString::number( Value ) ) )
		cb->insertItem( QString::number( Value ) );
	}

/**
 * Returns HistoryIntInput's Value.
 * @return 
 */
int HistoryIntInput::value()
	{
	registerValue();
	return Value;
	}

/**
 * Sets HistoryIntInput's MinValue and adjusts Value.
 * @param d 
 */
void HistoryIntInput::setMinValue( int d )
	{
	if( d > Value )
		{
		Value = d;
		lineEdit->setText( QString::number( Value ) );
		}
	
	MinValue = d;
	iv->setBottom( d );
	}

/**
 * Returns HistoryIntInput's MinValue.
 * You can also read this.MinValue .
 * @return 
 */
int HistoryIntInput::minValue()
	{
	return MinValue;
	}

/**
 * Sets HistoryIntInput's MaxValue and adjusts Value.
 * @param d 
 */
void HistoryIntInput::setMaxValue( int d )
	{
	if( d < Value )
		{
		Value = d;
		lineEdit->setText( QString::number( Value ) );
		}
	
	MaxValue = d;
	iv->setTop( d );
	}

/**
 * Returns HistoryIntInput's MaxValue.
 * You can also read this.MaxValue .
 * @return 
 */
int HistoryIntInput::maxValue()
	{
	return MaxValue;
	}

void HistoryIntInput::addValue( int d )
	{
	Value += d;
	
	if( Value > MaxValue )
		Value = MaxValue;
		
	lineEdit->setText( QString::number( Value ) );
	
	if(  ! cb->items().contains( QString::number( Value ) ) )
		cb->insertItem( QString::number( Value ) );
	}

void HistoryIntInput::subValue( int d )
	{
	Value -= d;
	
	if( Value < MinValue )
		Value = MinValue;
	
	lineEdit->setText( QString::number( Value ) );
	
	if(  ! cb->items().contains( QString::number( Value ) ) )
		cb->insertItem( QString::number( Value ) );
	}


#include "historyintinput.moc"




